"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = function (_ref) {
  var t = _ref.types;

  var buildScope = function buildScope(path) {
    var scope = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

    var functionParent = path.getFunctionParent();
    if (!functionParent) {
      return scope;
    }
    if (functionParent.isFunctionDeclaration()) {
      scope.push(functionParent.node.id.name);
    } else if (t.isClassProperty(functionParent) || t.isClassMethod(functionParent)) {
      scope.push(functionParent.node.key.name);
      var classBody = functionParent.findParent(function (path) {
        return path.isClassDeclaration() || path.isClassExpression();
      });
      if (classBody) {
        scope.push(classBody.node.id.name);
        return buildScope(classBody, scope);
      }
    } else if (functionParent.isArrowFunctionExpression()) {
      var arrFuncParent = functionParent.findParent(function (path) {
        return path;
      });
      if (t.isVariableDeclarator(arrFuncParent)) {
        scope.push(arrFuncParent.node.id.name);
      } else if (t.isClassProperty(arrFuncParent) || t.isClassMethod(arrFuncParent)) {
        scope.push(arrFuncParent.node.key.name);
        var _classBody = arrFuncParent.findParent(function (path) {
          return path.isClassDeclaration() || path.isClassExpression();
        });
        if (_classBody) {
          scope.push(_classBody.node.id.name);
          return buildScope(_classBody, scope);
        }
      } else if (t.isCallExpression(arrFuncParent)) {
        var callee = arrFuncParent.node.callee;

        scope.push("[" + callee.object.name + "." + callee.property.name + "]");
      }
    }
    return buildScope(functionParent, scope);
  };

  var injectVariableNames = function injectVariableNames() {
    var args = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

    return args.reduce(function (acc, arg) {
      if (t.isIdentifier(arg)) {
        return [].concat(_toConsumableArray(acc), [t.stringLiteral(arg.name), arg]);
      }
      return [].concat(_toConsumableArray(acc), [arg]);
    }, []);
  };

  var buildSettings = function buildSettings(opts) {
    var methods = opts.methods,
        flags = _objectWithoutProperties(opts, ["methods"]);
    // output speards the flags over each method
    // in the future this could be expanded to allow method level config


    return (methods || defaultMethods).reduce(function (acc, curr) {
      return _extends({}, acc, _defineProperty({}, curr, Object.values(flags).length ? flags : defaultSettings));
    }, {});
  };

  var isConsoleStatement = function isConsoleStatement(path) {
    return path.get("object").isIdentifier({ name: "console" });
  };

  return {
    name: "babel-plugin-captains-log", // not required
    visitor: {
      MemberExpression: function MemberExpression(path, _ref2) {
        var opts = _ref2.opts;

        if (!isConsoleStatement(path)) {
          return;
        }
        var settings = buildSettings(opts || {});
        if (!Object.keys(settings).includes(path.node.property.name)) {
          return;
        }
        var options = settings[path.node.property.name];
        // console.log(options);
        var parent = path.parent;
        if (t.isCallExpression(parent) && parent.arguments && Array.isArray(parent.arguments)) {
          // add variable names
          if (options.injectVariableName) {
            parent.arguments = injectVariableNames(parent.arguments);
          }
          // prepend console statement scope
          if (options.injectScope) {
            var scope = buildScope(path);
            parent.arguments.unshift(t.stringLiteral(scope.reverse().join(".") + ":"));
          }
        }
      }
    }
  };
};

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var defaultSettings = {
  injectScope: true,
  injectVariableName: true
};

var defaultMethods = ["debug", "error", "exception", "info", "log", "warn"];